@@ -23,16 +23,15 @@ import pkg_resources
 from AnyQt.QtWidgets import (
     QMainWindow, QWidget, QAction, QActionGroup, QMenu, QMenuBar, QDialog,
     QFileDialog, QMessageBox, QVBoxLayout, QSizePolicy, QToolBar, QToolButton,
-    QDockWidget, QApplication, QShortcut, QPlainTextEdit,
-    QPlainTextDocumentLayout, QFileIconProvider
+    QDockWidget, QApplication, QShortcut, QFileIconProvider
 )
 from AnyQt.QtGui import (
-    QColor, QIcon, QDesktopServices, QKeySequence, QTextDocument,
+    QColor, QIcon, QDesktopServices, QKeySequence,
     QWhatsThisClickedEvent, QShowEvent, QCloseEvent
 )
 from AnyQt.QtCore import (
     Qt, QObject, QEvent, QSize, QUrl, QFile, QByteArray, QFileInfo,
-    QSettings, QStandardPaths, QAbstractItemModel, QT_VERSION)
+    QSettings, QStandardPaths, QAbstractItemModel, QMimeData, QT_VERSION)
 
 try:
     from AnyQt.QtWebEngineWidgets import QWebEngineView
@@ -70,6 +69,7 @@ from .settings import UserSettingsDialog, category_state
 from ..document.schemeedit import SchemeEditWidget
 from ..document.quickmenu import QuickMenu
 from ..document.commands import UndoCommand
+from ..document import interactions
 from ..gui.itemmodels import FilterProxyModel
 from ..registry import WidgetRegistry, WidgetDescription, CategoryDescription
 from ..registry.qt import QtWidgetRegistry
@@ -77,7 +77,7 @@ from ..utils.settings import QSettings_readArray, QSettings_writeArray
 from ..utils.qinvoke import qinvoke
 from ..utils.pickle import Pickler, Unpickler, glob_scratch_swps, swp_name, \
     canvas_scratch_name_memo, register_loaded_swp
-from ..utils import unique, group_by_all, set_flag
+from ..utils import unique, group_by_all, set_flag, findf
 from ..utils.asyncutils import get_event_loop
 from ..utils.qobjref import qobjref
 
@@ -164,7 +164,6 @@ class CanvasMainWindow(QMainWindow):
         self.__registry_model = None  # type: Optional[QAbstractItemModel]
         # Proxy widget registry model
         self.__proxy_model = None  # type: Optional[FilterProxyModel]
-        self.__streams = []  # type: List[TextStream]
 
         # TODO: Help view and manager to separate singleton instance.
         self.help = None  # type: HelpManager
@@ -223,6 +222,7 @@ class CanvasMainWindow(QMainWindow):
         w.layout().setContentsMargins(20, 0, 10, 0)
 
         self.scheme_widget = SchemeEditWidget()
+        self.scheme_widget.setDropHandlers([interactions.PluginDropHandler(),])
         self.set_scheme(config.workflow_constructor(parent=self))
 
         # Save crash recovery swap file on changes to workflow
@@ -231,7 +231,7 @@ class CanvasMainWindow(QMainWindow):
         dropfilter = UrlDropEventFilter(self)
         dropfilter.urlDropped.connect(self.open_scheme_file)
         self.scheme_widget.setAcceptDrops(True)
-        self.scheme_widget.installEventFilter(dropfilter)
+        self.scheme_widget.view().viewport().installEventFilter(dropfilter)
 
         w.layout().addWidget(self.scheme_widget)
 
@@ -528,8 +528,8 @@ class CanvasMainWindow(QMainWindow):
             objectName="recent-action",
             toolTip=self.tr("Browse and open a recent workflow."),
             triggered=self.recent_scheme,
-            shortcut=QKeySequence(Qt.ControlModifier |
-                                  (Qt.ShiftModifier | Qt.Key_R)),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.ShiftModifier |
+                                  Qt.Key_R),
             icon=canvas_icons("Recent.svg")
         )
         self.reload_last_action = QAction(
@@ -601,8 +601,8 @@ class CanvasMainWindow(QMainWindow):
             self.tr("Expand Tool Dock"), self,
             objectName="toggle-tool-dock-expand",
             checkable=True,
-            shortcut=QKeySequence(Qt.ControlModifier |
-                                  (Qt.ShiftModifier | Qt.Key_D)),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.ShiftModifier |
+                                  Qt.Key_D),
             triggered=self.set_tool_dock_expanded
         )
         self.toggle_tool_dock_expand.setChecked(True)
@@ -929,7 +929,7 @@ class CanvasMainWindow(QMainWindow):
                 popup.setRootIndex(model.index(i, 0))
                 button = self.quick_category.buttonForAction(action)
                 pos = popup_position_from_source(popup, button)
-                action = popup.exec_(pos)
+                action = popup.exec(pos)
                 if action is not None:
                     self.on_tool_box_widget_activated(action)
 
@@ -1017,28 +1017,24 @@ class CanvasMainWindow(QMainWindow):
                             self.SETTINGS_VERSION)
         window.set_tool_dock_expanded(self.dock_widget.expanded())
         window.set_float_widgets_on_top_enabled(self.float_widgets_on_top_action.isChecked())
-        logview = window.output_view()  # type: OutputView
 
-        te = logview.findChild(QPlainTextEdit)
+        output = window.output_view()  # type: OutputView
+        doc = self.output_view().document()
+        doc = doc.clone(output)
+        output.setDocument(doc)
 
-        doc = self.output_view().findChild(QPlainTextEdit).document()
-        # first clone the existing document and set it on the new instance
-        doc = doc.clone(parent=te)  # type: QTextDocument
-        doc.setDocumentLayout(QPlainTextDocumentLayout(doc))
-        te.setDocument(doc)
+        def is_connected(stream: TextStream) -> bool:
+            item = findf(doc.connectedStreams(), lambda s: s is stream)
+            return item is not None
 
-        # route the stdout/err if possible
+        # # route the stdout/err if possible
+        # TODO: Deprecate and remove this behaviour (use connectStream)
         stdout, stderr = sys.stdout, sys.stderr
-        if isinstance(stdout, TextStream):
-            stdout.stream.connect(logview.write)
+        if isinstance(stdout, TextStream) and not is_connected(stdout):
+            doc.connectStream(stdout)
 
-        if isinstance(stderr, TextStream):
-            err_formatter = logview.formatted(color=Qt.red)
-            stderr.stream.connect(err_formatter.write)
-
-        for stream in self.__streams:
-            if stream not in (stdout, stderr):
-                window.connect_output_stream(stream)
+        if isinstance(stderr, TextStream) and not is_connected(stderr):
+            doc.connectStream(stderr, color=Qt.red)
 
         CanvasMainWindow._instances.append(window)
         window.destroyed.connect(
@@ -1238,7 +1234,8 @@ class CanvasMainWindow(QMainWindow):
         workflow: Optional[Scheme]
         """
         new_scheme = config.workflow_constructor(parent=self)
-        new_scheme.set_runtime_env("basedir", os.path.dirname(path))
+        new_scheme.set_runtime_env(
+            "basedir", os.path.abspath(os.path.dirname(path)))
         errors = []  # type: List[Exception]
         try:
             new_scheme.load_from(
@@ -1512,7 +1509,7 @@ class CanvasMainWindow(QMainWindow):
         # existing scheme file if `scheme.save_to` raises an error.
         buffer = io.BytesIO()
         try:
-            scheme.set_runtime_env("basedir", dirname)
+            scheme.set_runtime_env("basedir", os.path.abspath(dirname))
             scheme.save_to(buffer, pretty=True, pickle_fallback=True)
         except Exception:
             log.error("Error saving %r to %r", scheme, filename, exc_info=True)
@@ -1804,7 +1801,7 @@ class CanvasMainWindow(QMainWindow):
 
         model.delayedScanUpdate()
 
-        status = dialog.exec_()
+        status = dialog.exec()
 
         index = dialog.currentIndex()
 
@@ -1838,7 +1835,7 @@ class CanvasMainWindow(QMainWindow):
         dialog.setModel(model)
 
         model.delayedScanUpdate()
-        status = dialog.exec_()
+        status = dialog.exec()
         index = dialog.currentIndex()
 
         dialog.deleteLater()
@@ -1904,8 +1901,8 @@ class CanvasMainWindow(QMainWindow):
             objectName="welcome-recent-action",
             toolTip=self.tr("Browse and open a recent workflow."),
             triggered=open_recent,
-            shortcut=QKeySequence(Qt.ControlModifier |
-                                  (Qt.ShiftModifier | Qt.Key_R)),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.ShiftModifier |
+                                  Qt.Key_R),
             icon=canvas_icons("Recent.svg")
         )
 
@@ -1934,7 +1931,7 @@ class CanvasMainWindow(QMainWindow):
             settings.value("startup/show-welcome-screen", True, type=bool)
         )
 
-        status = dialog.exec_()
+        status = dialog.exec()
 
         settings.setValue("startup/show-welcome-screen",
                           dialog.showAtStartup())
@@ -1972,7 +1969,7 @@ class CanvasMainWindow(QMainWindow):
         dlg = self.scheme_properties_dialog()
         dlg.setAutoCommit(False)
         dlg.setScheme(scheme)
-        status = dlg.exec_()
+        status = dlg.exec()
 
         if status == QDialog.Accepted:
             editor = dlg.editor
@@ -2030,7 +2027,7 @@ class CanvasMainWindow(QMainWindow):
         dlg = UserSettingsDialog(self)
         dlg.setWindowTitle(self.tr("Preferences"))
         dlg.show()
-        status = dlg.exec_()
+        status = dlg.exec()
 
         if status == 0:
             self.user_preferences_changed_notify_all()
@@ -2058,14 +2055,14 @@ class CanvasMainWindow(QMainWindow):
                         "Insufficient permissions to install add-ons. Try starting {name} "
                         "as a system administrator or install {name} in user folders."
                         .format(name=name),
-                        parent=self).exec_()
+                        parent=self).exec()
         dlg = addons.AddonManagerDialog(
             self, windowTitle=self.tr("Installer"), modal=True
         )
         dlg.setStyle(QApplication.style())
         dlg.setAttribute(Qt.WA_DeleteOnClose)
         dlg.start(config.default)
-        return dlg.exec_()
+        return dlg.exec()
 
     def set_float_widgets_on_top_enabled(self, enabled):
         # type: (bool) -> None
@@ -2088,7 +2085,7 @@ class CanvasMainWindow(QMainWindow):
         """
         dlg = AboutDialog(self)
         dlg.setAttribute(Qt.WA_DeleteOnClose)
-        dlg.exec_()
+        dlg.exec()
 
     def add_recent_scheme(self, title, path):
         # type: (str, str) -> None
@@ -2495,18 +2492,16 @@ class CanvasMainWindow(QMainWindow):
         The `stream` will be 'inherited' by new windows created by
         `create_new_window`.
         """
-        self.__streams.append(stream)
-        view = self.output_view()
-        stream.stream.connect(view.write)
+        doc = self.output_view().document()
+        doc.connectStream(stream)
 
     def disconnect_output_stream(self, stream: TextStream):
         """
         Disconnect a :class:`TextStream` instance from this window's
         output view.
         """
-        self.__streams.remove(stream)
-        view = self.output_view()
-        stream.stream.disconnect(view.write)
+        doc = self.output_view().document()
+        doc.disconnectStream(stream)
 
 
 def updated_flags(flags, mask, state):
@@ -2547,27 +2542,29 @@ def category_filter_function(state):
 class UrlDropEventFilter(QObject):
     urlDropped = Signal(QUrl)
 
+    def acceptsDrop(self, mime: QMimeData) -> bool:
+        if mime.hasUrls() and len(mime.urls()) == 1:
+            url = mime.urls()[0]
+            if url.scheme() == "file":
+                filename = url.toLocalFile()
+                _, ext = os.path.splitext(filename)
+                if ext == ".ows":
+                    return True
+        return False
+
     def eventFilter(self, obj, event):
         etype = event.type()
         if etype == QEvent.DragEnter or etype == QEvent.DragMove:
-            mime = event.mimeData()
-            if mime.hasUrls() and len(mime.urls()) == 1:
-                url = mime.urls()[0]
-                if url.scheme() == "file":
-                    filename = url.toLocalFile()
-                    _, ext = os.path.splitext(filename)
-                    if ext == ".ows":
-                        event.acceptProposedAction()
-                        return True
-
-        elif etype == QEvent.Drop:
-            mime = event.mimeData()
-            urls = mime.urls()
-            if urls:
-                url = urls[0]
-                self.urlDropped.emit(url)
+            if self.acceptsDrop(event.mimeData()):
+                event.acceptProposedAction()
                 return True
-
+        elif etype == QEvent.Drop:
+            if self.acceptsDrop(event.mimeData()):
+                urls = event.mimeData().urls()
+                if urls:
+                    url = urls[0]
+                    self.urlDropped.emit(url)
+                    return True
         return super().eventFilter(obj, event)
 
 
