@@ -11,16 +11,19 @@ import typing
 from typing import List, Tuple, Optional, Any
 
 from AnyQt.QtWidgets import (
-    QGraphicsItem, QGraphicsPathItem, QGraphicsWidget, QGraphicsTextItem,
-    QGraphicsDropShadowEffect, QGraphicsSceneHoverEvent,
+    QGraphicsItem, QGraphicsPathItem, QGraphicsWidget,
+    QGraphicsDropShadowEffect, QGraphicsSceneHoverEvent, QStyle,
+    QGraphicsSceneMouseEvent
 )
 from AnyQt.QtGui import (
-    QPen, QBrush, QColor, QPainterPath, QTransform, QPalette,
+    QPen, QBrush, QColor, QPainterPath, QTransform, QPalette, QFont,
 )
 from AnyQt.QtCore import Qt, QPointF, QRectF, QLineF, QEvent, QPropertyAnimation, Signal, QTimer
 
 from .nodeitem import AnchorPoint, SHADOW_COLOR
+from .graphicstextitem import GraphicsTextItem
 from .utils import stroke_path
+from ...registry import InputSignal, OutputSignal
 
 from ...scheme import SchemeLink
 
@@ -41,6 +44,7 @@ class LinkCurveItem(QGraphicsPathItem):
         self.__animationEnabled = False
         self.__hover = False
         self.__enabled = True
+        self.__selected = False
         self.__shape = None  # type: Optional[QPainterPath]
         self.__curvepath = QPainterPath()
         self.__curvepath_disabled = None  # type: Optional[QPainterPath]
@@ -73,9 +77,17 @@ class LinkCurveItem(QGraphicsPathItem):
 
     def setHoverState(self, state):
         # type: (bool) -> None
-        self.prepareGeometryChange()
-        self.__hover = state
-        self.__update()
+        if self.__hover != state:
+            self.prepareGeometryChange()
+            self.__hover = state
+            self.__update()
+
+    def setSelectionState(self, state):
+        # type: (bool) -> None
+        if self.__selected != state:
+            self.prepareGeometryChange()
+            self.__selected = state
+            self.__update()
 
     def setLinkEnabled(self, state):
         # type: (bool) -> None
@@ -120,14 +132,13 @@ class LinkCurveItem(QGraphicsPathItem):
 
     def __update(self):
         # type: () -> None
-        radius = 5 if self.__hover else 0
-
+        radius = 5 if self.__hover or self.__selected else 0
         if radius != 0 and not self.shadow.isEnabled():
             self.shadow.setEnabled(True)
 
         if self.__animationEnabled:
             if self.__blurAnimation.state() == QPropertyAnimation.Running:
-                self.__blurAnimation.pause()
+                self.__blurAnimation.stop()
 
             self.__blurAnimation.setStartValue(self.shadow.blurRadius())
             self.__blurAnimation.setEndValue(radius)
@@ -294,7 +305,7 @@ _State = SchemeLink.State
 
 class LinkItem(QGraphicsWidget):
     """
-    A Link item in the canvas that connects two :class:`.NodeItem`\s in the
+    A Link item in the canvas that connects two :class:`.NodeItem`\\s in the
     canvas.
 
     The link curve connects two `Anchor` items (see :func:`setSourceItem`
@@ -308,6 +319,9 @@ class LinkItem(QGraphicsWidget):
     #: Signal emitted when the item has been activated (double-click)
     activated = Signal()
 
+    #: Signal emitted the the item's selection state changes.
+    selectedChanged = Signal(bool)
+
     #: Z value of the item
     Z_VALUE = 0
 
@@ -332,6 +346,7 @@ class LinkItem(QGraphicsWidget):
         super().__init__(parent, **kwargs)
         self.setAcceptedMouseButtons(Qt.RightButton | Qt.LeftButton)
         self.setAcceptHoverEvents(True)
+        self.__animationEnabled = False
 
         self.setZValue(self.Z_VALUE)
 
@@ -342,7 +357,7 @@ class LinkItem(QGraphicsWidget):
 
         self.curveItem = LinkCurveItem(self)
 
-        self.linkTextItem = QGraphicsTextItem(self)
+        self.linkTextItem = GraphicsTextItem(self)
         self.linkTextItem.setAcceptedMouseButtons(Qt.NoButton)
         self.linkTextItem.setAcceptHoverEvents(False)
         self.__sourceName = ""
@@ -351,15 +366,19 @@ class LinkItem(QGraphicsWidget):
         self.__dynamic = False
         self.__dynamicEnabled = False
         self.__state = LinkItem.NoState
+        self.__channelNamesVisible = True
         self.hover = False
 
+        self.channelNameAnim = QPropertyAnimation(self.linkTextItem, b'opacity', self)
+        self.channelNameAnim.setDuration(50)
+
         self.prepareGeometryChange()
         self.__updatePen()
         self.__updatePalette()
         self.__updateFont()
 
-    def setSourceItem(self, item, anchor=None):
-        # type: (Optional[NodeItem], Optional[AnchorPoint]) -> None
+    def setSourceItem(self, item, signal=None, anchor=None):
+        # type: (Optional[NodeItem], Optional[OutputSignal], Optional[AnchorPoint]) -> None
         """
         Set the source `item` (:class:`.NodeItem`). Use `anchor`
         (:class:`.AnchorPoint`) as the curve start point (if ``None`` a new
@@ -381,14 +400,17 @@ class LinkItem(QGraphicsWidget):
 
                 if self.sourceItem is not None:
                     self.sourceItem.removeOutputAnchor(self.sourceAnchor)
-
+                    self.sourceItem.selectedChanged.disconnect(
+                        self.__updateSelectedState)
                 self.sourceItem = self.sourceAnchor = None
 
             self.sourceItem = item
 
             if item is not None and anchor is None:
                 # Create a new output anchor for the item if none is provided.
-                anchor = item.newOutputAnchor()
+                anchor = item.newOutputAnchor(signal)
+            if item is not None:
+                item.selectedChanged.connect(self.__updateSelectedState)
 
         if anchor != self.sourceAnchor:
             if self.sourceAnchor is not None:
@@ -405,8 +427,8 @@ class LinkItem(QGraphicsWidget):
 
         self.__updateCurve()
 
-    def setSinkItem(self, item, anchor=None):
-        # type: (Optional[NodeItem], Optional[AnchorPoint]) -> None
+    def setSinkItem(self, item, signal=None, anchor=None):
+        # type: (Optional[NodeItem], Optional[InputSignal], Optional[AnchorPoint]) -> None
         """
         Set the sink `item` (:class:`.NodeItem`). Use `anchor`
         (:class:`.AnchorPoint`) as the curve end point (if ``None`` a new
@@ -428,14 +450,17 @@ class LinkItem(QGraphicsWidget):
 
                 if self.sinkItem is not None:
                     self.sinkItem.removeInputAnchor(self.sinkAnchor)
-
+                    self.sinkItem.selectedChanged.disconnect(
+                        self.__updateSelectedState)
                 self.sinkItem = self.sinkAnchor = None
 
             self.sinkItem = item
 
             if item is not None and anchor is None:
                 # Create a new input anchor for the item if none is provided.
-                anchor = item.newInputAnchor()
+                anchor = item.newInputAnchor(signal)
+            if item is not None:
+                item.selectedChanged.connect(self.__updateSelectedState)
 
         if self.sinkAnchor != anchor:
             if self.sinkAnchor is not None:
@@ -457,7 +482,9 @@ class LinkItem(QGraphicsWidget):
         """
         Set the visibility of the channel name text.
         """
-        self.linkTextItem.setVisible(visible)
+        if self.__channelNamesVisible != visible:
+            self.__channelNamesVisible = visible
+        self.__initChannelNameOpacity()
 
     def setSourceName(self, name):
         # type: (str) -> None
@@ -496,6 +523,8 @@ class LinkItem(QGraphicsWidget):
         """
         Set the link item animation enabled state.
         """
+        if self.__animationEnabled != enabled:
+            self.__animationEnabled = enabled
         self.curveItem.setAnimationEnabled(enabled)
 
     def _sinkPosChanged(self, *arg):
@@ -553,8 +582,9 @@ class LinkItem(QGraphicsWidget):
         else:
             text = ""
 
-        self.linkTextItem.setHtml('<div align="center">{0}</div>'
-                                  .format(text))
+        self.linkTextItem.setHtml(
+            '<div align="center" style="font-size: small" >{0}</div>'
+            .format(text))
         path = self.curveItem.curvePath()
 
         # Constrain the text width if it is too long to fit on a single line
@@ -586,7 +616,12 @@ class LinkItem(QGraphicsWidget):
 
             transform = QTransform()
             transform.translate(center.x(), center.y())
-            transform.rotate(-angle)
+
+            # Rotate text to be on top of link
+            if 90 <= angle < 270:
+                transform.rotate(180 - angle)
+            else:
+                transform.rotate(-angle)
 
             # Center and move above the curve path.
             transform.translate(-brect.width() / 2, -brect.height())
@@ -611,6 +646,21 @@ class LinkItem(QGraphicsWidget):
                 self.sourceAnchor.setHoverState(state)
             self.curveItem.setHoverState(state)
             self.__updatePen()
+            self.__updateChannelNameVisibility()
+            self.__updateZValue()
+
+    def __updateZValue(self):
+        text_ss = self.linkTextItem.styleState()
+        if self.hover:
+            text_ss |= QStyle.State_HasFocus
+            z = 9999
+            self.linkTextItem.setParentItem(None)
+        else:
+            text_ss &= ~QStyle.State_HasFocus
+            z = self.Z_VALUE
+            self.linkTextItem.setParentItem(self)
+        self.linkTextItem.setZValue(z)
+        self.linkTextItem.setStyleState(text_ss)
 
     def mouseDoubleClickEvent(self, event):
         # type: (QGraphicsSceneMouseEvent) -> None
@@ -633,6 +683,26 @@ class LinkItem(QGraphicsWidget):
         self.curveItem.removeSceneEventFilter(self)
         return super().hoverLeaveEvent(event)
 
+    def __initChannelNameOpacity(self):
+        if self.__channelNamesVisible:
+            self.linkTextItem.setOpacity(1)
+        else:
+            self.linkTextItem.setOpacity(0)
+
+    def __updateChannelNameVisibility(self):
+        if self.__channelNamesVisible:
+            return
+        enabled = self.hover or self.isSelected() or self.__isSelectedImplicit()
+        targetOpacity = 1 if enabled else 0
+        if not self.__animationEnabled:
+            self.linkTextItem.setOpacity(targetOpacity)
+        else:
+            if self.channelNameAnim.state() == QPropertyAnimation.Running:
+                self.channelNameAnim.stop()
+            self.channelNameAnim.setStartValue(self.linkTextItem.opacity())
+            self.channelNameAnim.setEndValue(targetOpacity)
+            self.channelNameAnim.start()
+
     def changeEvent(self, event):
         # type: (QEvent) -> None
         if event.type() == QEvent.PaletteChange:
@@ -728,15 +798,7 @@ class LinkItem(QGraphicsWidget):
         """
         if self.__state != state:
             self.__state = state
-
-            if state & LinkItem.Pending:
-                self.sinkAnchor.setBrush(QBrush(Qt.red))
-            else:
-                self.sinkAnchor.setBrush(QBrush(QColor("#9CACB4")))
-            if state & LinkItem.Invalidated:
-                self.sourceAnchor.setBrush(QBrush(Qt.red))
-            else:
-                self.sourceAnchor.setBrush(QBrush(QColor("#9CACB4")))
+            self.__updateAnchors()
             self.__updatePen()
 
     def runtimeState(self):
@@ -767,7 +829,7 @@ class LinkItem(QGraphicsWidget):
         normal.setStyle(pen_style)
         hover.setStyle(pen_style)
 
-        if self.hover:
+        if self.hover or self.isSelected():
             pen = hover
         else:
             pen = normal
@@ -781,4 +843,41 @@ class LinkItem(QGraphicsWidget):
 
     def __updateFont(self):
         # type: () -> None
-        self.linkTextItem.setFont(self.font())
+        font = self.font()
+        # linkTextItem will be rotated. Hinting causes bad positioning under
+        # rotation so we prefer to disable it. This is only a hint, on windows
+        # (DirectWrite engine) vertical hinting is still performed.
+        font.setHintingPreference(QFont.PreferNoHinting)
+        self.linkTextItem.setFont(font)
+
+    def __updateAnchors(self):
+        state = QStyle.State(0)
+        if self.hover:
+            state |= QStyle.State_MouseOver
+        if self.isSelected() or self.__isSelectedImplicit():
+            state |= QStyle.State_Selected
+        if self.sinkAnchor is not None:
+            self.sinkAnchor.indicator.setStyleState(state)
+            self.sinkAnchor.indicator.setLinkState(self.__state)
+        if self.sourceAnchor is not None:
+            self.sourceAnchor.indicator.setStyleState(state)
+            self.sourceAnchor.indicator.setLinkState(self.__state)
+
+    def __updateSelectedState(self):
+        selected = self.isSelected() or self.__isSelectedImplicit()
+        self.linkTextItem.setSelectionState(selected)
+        self.__updatePen()
+        self.__updateAnchors()
+        self.__updateChannelNameVisibility()
+        self.curveItem.setSelectionState(selected)
+
+    def __isSelectedImplicit(self):
+        source, sink = self.sourceItem, self.sinkItem
+        return (source is not None and source.isSelected()
+                and sink is not None and sink.isSelected())
+
+    def itemChange(self, change: QGraphicsItem.GraphicsItemChange, value: Any) -> Any:
+        if change == QGraphicsItem.ItemSelectedHasChanged:
+            self.__updateSelectedState()
+            self.selectedChanged.emit(value)
+        return super().itemChange(change, value)
