@@ -7,17 +7,18 @@ import traceback
 import ctypes
 
 from contextlib import contextmanager
+from typing import Optional, Union
 
 from AnyQt.QtWidgets import (
-    QWidget, QMessageBox, QStyleOption, QStyle, QTextEdit
+    QWidget, QMessageBox, QStyleOption, QStyle, QTextEdit, QScrollBar
 )
 from AnyQt.QtGui import (
     QGradient, QLinearGradient, QRadialGradient, QBrush, QPainter,
-    QPaintEvent, QColor, QPixmap, QPixmapCache, QTextOption
+    QPaintEvent, QColor, QPixmap, QPixmapCache, QTextOption, QGuiApplication,
+    QTextCharFormat, QFont
 )
-from AnyQt.QtCore import Qt, QPointF, QPoint, QRect, QRectF
-
-import sip
+from AnyQt.QtCore import Qt, QPointF, QPoint, QRect, QRectF, Signal, QEvent
+from AnyQt import sip
 
 
 @contextmanager
@@ -59,6 +60,33 @@ def disabled(qobject):
         qobject.setEnabled(old_state)
 
 
+@contextmanager
+def disconnected(signal, slot, type=Qt.UniqueConnection):
+    """
+    A context manager disconnecting a slot from a signal.
+    ::
+
+        with disconnected(scene.selectionChanged, self.onSelectionChanged):
+            # Can change item selection in a scene without
+            # onSelectionChanged being invoked.
+            do_something()
+
+    Warning
+    -------
+    The relative order of the slot in signal's connections is not preserved.
+
+    Raises
+    ------
+    TypeError:
+        If the slot was not connected to the signal
+    """
+    signal.disconnect(slot)
+    try:
+        yield
+    finally:
+        signal.connect(slot, type)
+
+
 def StyledWidget_paintEvent(self, event):
     # type: (QWidget, QPaintEvent) -> None
     """A default styled QWidget subclass  paintEvent function.
@@ -75,6 +103,16 @@ class StyledWidget(QWidget):
     paintEvent = StyledWidget_paintEvent  # type: ignore
 
 
+class ScrollBar(QScrollBar):
+    #: Emitted when the scroll bar receives a StyleChange event
+    styleChange = Signal()
+
+    def changeEvent(self, event: QEvent) -> None:
+        if event.type() == QEvent.StyleChange:
+            self.styleChange.emit()
+        super().changeEvent(event)
+
+
 def is_transparency_supported():  # type: () -> bool
     """Is window transparency supported by the current windowing system.
     """
@@ -307,7 +345,7 @@ def message(icon, text, title=None, informative_text=None, details=None,
     if default_button is not None:
         mbox.setDefaultButton(default_button)
 
-    return mbox.exec_()
+    return mbox.exec()
 
 
 def innerGlowBackgroundPixmap(color, size, radius=5):
@@ -341,7 +379,7 @@ def innerGlowBackgroundPixmap(color, size, radius=5):
     # draw radial gradient pixmap
     pixPainter = QPainter(gradientPixmap)
     pixPainter.setPen(Qt.NoPen)
-    gradient = QRadialGradient(center, radius - 1)
+    gradient = QRadialGradient(QPointF(center), radius - 1)
     gradient.setColorAt(0, light_color)
     gradient.setColorAt(1, dark_color)
     pixPainter.setBrush(gradient)
@@ -530,3 +568,112 @@ def innerShadowPixmap(color, size, pos, length=5):
     QPixmapCache.insert(key, finalShadow)
 
     return finalShadow
+
+
+def clipboard_has_format(mimetype):
+    # type: (str) -> bool
+    """Does the system clipboard contain data for mimetype?"""
+    cb = QGuiApplication.clipboard()
+    if cb is None:
+        return False
+    mime = cb.mimeData()
+    if mime is None:
+        return False
+    return mime.hasFormat(mimetype)
+
+
+def clipboard_data(mimetype: str) -> Optional[bytes]:
+    """Return the binary data of the system clipboard for mimetype."""
+    cb = QGuiApplication.clipboard()
+    if cb is None:
+        return None
+    mime = cb.mimeData()
+    if mime is None:
+        return None
+    if mime.hasFormat(mimetype):
+        return bytes(mime.data(mimetype))
+    else:
+        return None
+
+
+_Color = Union[QColor, QBrush, Qt.GlobalColor, QGradient]
+
+
+def update_char_format(
+        baseformat: QTextCharFormat,
+        color: Optional[_Color] = None,
+        background: Optional[_Color] = None,
+        weight: Optional[int] = None,
+        italic: Optional[bool] = None,
+        underline: Optional[bool] = None,
+        font: Optional[QFont] = None
+) -> QTextCharFormat:
+    """
+    Return a copy of `baseformat` :class:`QTextCharFormat` with
+    updated color, weight, background and font properties.
+    """
+    charformat = QTextCharFormat(baseformat)
+    if color is not None:
+        charformat.setForeground(color)
+    if background is not None:
+        charformat.setBackground(background)
+    if font is not None:
+        assert weight is None and italic is None and underline is None
+        charformat.setFont(font)
+    else:
+        if weight is not None:
+            charformat.setFontWeight(weight)
+        if italic is not None:
+            charformat.setFontItalic(italic)
+        if underline is not None:
+            charformat.setFontUnderline(underline)
+    return charformat
+
+
+def update_font(
+        basefont: QFont,
+        weight: Optional[int] = None,
+        italic: Optional[bool] = None,
+        underline: Optional[bool] = None,
+        pixelSize: Optional[int] = None,
+        pointSize: Optional[float] = None
+) -> QFont:
+    """
+    Return a copy of `basefont` :class:`QFont` with updated properties.
+    """
+    font = QFont(basefont)
+
+    if weight is not None:
+        font.setWeight(weight)
+
+    if italic is not None:
+        font.setItalic(italic)
+
+    if underline is not None:
+        font.setUnderline(underline)
+
+    if pixelSize is not None:
+        font.setPixelSize(pixelSize)
+
+    if pointSize is not None:
+        font.setPointSizeF(pointSize)
+
+    return font
+
+
+def screen_geometry(widget: QWidget, pos: Optional[QPoint] = None) -> QRect:
+    screen = widget.screen()
+    if pos is not None:
+        sibling = screen.virtualSibling(pos)
+        if sibling is not None:
+            screen = sibling
+    return screen.geometry()
+
+
+def available_screen_geometry(widget: QWidget, pos: Optional[QPoint] = None) -> QRect:
+    screen = widget.screen()
+    if pos is not None:
+        sibling = screen.virtualSibling(pos)
+        if sibling is not None:
+            screen = sibling
+    return screen.availableGeometry()
