@@ -93,9 +93,12 @@ class Scheme(QObject):
     # Signal emitted when the description of scheme changes.
     description_changed = Signal(str)
 
-    #: Signal emitted when the associated runtime environment changes
+    # Signal emitted when the associated runtime environment changes
     runtime_env_changed = Signal(str, object, object)
 
+    # Signal emitted by subclass upon a detected settings change
+    node_properties_changed = Signal()
+
     def __init__(self, parent=None, title="", description="", env={},
                  **kwargs):
         # type: (Optional[QObject], str, str, Mapping[str, Any], Any) -> None
